<?php

declare(strict_types=1);

namespace FiloBlu\Refilo\Plugins\Helper;

use FiloBlu\Refilo\Remote\Entity\UrlRewrite;
use Magento\Catalog\Model\CategoryFactory;
use Magento\Framework\ObjectManagerInterface;
use Magento\Store\Model\StoreManagerInterface;
use MageWorx\SeoBase\Api\CustomCanonicalRepositoryInterface;

use function in_array;

/**
 *
 */
class Url
{
    /**
     * @var \Magento\Framework\ObjectManagerInterface
     */
    private $objectManager;

    /**
     * @var \MageWorx\SeoBase\Api\CustomCanonicalRepositoryInterface|null
     */
    private $customUrlRepository;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    private $storeManager;
    /**
     * @var \Magento\Catalog\Model\CategoryFactory
     */
    private $categoryFactory;

    /**
     * @param \Magento\Catalog\Model\CategoryFactory $categoryFactory
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Framework\ObjectManagerInterface $objectManager
     */
    public function __construct(
        CategoryFactory $categoryFactory,
        StoreManagerInterface $storeManager,
        ObjectManagerInterface $objectManager
    ) {
        $this->objectManager = $objectManager;
        $this->storeManager = $storeManager;
        $this->categoryFactory = $categoryFactory;
    }

    /**
     * @param \FiloBlu\Refilo\Helper\Url $subject
     * @param string|null $result
     * @param \FiloBlu\Refilo\Remote\Entity\UrlRewrite $urlRewrite
     * @return string|null
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function afterGetCanonicalUrl(\FiloBlu\Refilo\Helper\Url $subject, $result, UrlRewrite $urlRewrite)
    {
        if (!$result || !in_array($urlRewrite->getType(), ['category', 'product'])) {
            return $result;
        }

        $repository = $this->getCustomCanonicalRepository();

        if ($repository === null) {
            return $result;
        }

        $storeId = $this->storeManager->getStore($urlRewrite->getStore())->getId();
        $customCanonical = $repository->getBySourceEntityData(
            $urlRewrite->getType(),
            $urlRewrite->getEntityId(),
            $storeId,
            false
        );

        if ($customCanonical) {
            $canonicalStoreId = null;
            $canonicalUrl = $repository->getCustomCanonicalUrl(
                $customCanonical,
                $storeId,
                $canonicalStoreId,
                true
            );

            if ($canonicalUrl) {
                return $subject->getStoreUrl($storeId, $canonicalUrl);
            }
        }

        return $result;
    }

    /**
     * @return \MageWorx\SeoBase\Api\CustomCanonicalRepositoryInterface|null
     */
    public function getCustomCanonicalRepository()
    {
        if ($this->customUrlRepository) {
            return $this->customUrlRepository;
        }

        if (!interface_exists(CustomCanonicalRepositoryInterface::class)) {
            return null;
        }

        $this->customUrlRepository = $this->objectManager->get(CustomCanonicalRepositoryInterface::class);
        return $this->customUrlRepository;
    }

    /**
     * @param \FiloBlu\Refilo\Helper\Url $subject
     * @param array $result
     * @param \FiloBlu\Refilo\Remote\Entity\UrlRewrite $urlRewrite
     * @return array
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function afterGetHreflang(\FiloBlu\Refilo\Helper\Url $subject, $result, UrlRewrite $urlRewrite)
    {
        if ($urlRewrite->getType() !== 'category') {
            return $result;
        }

        $categoryId = $urlRewrite->getEntityId();
        $urlRewriteStoreCode = $urlRewrite->getStore();

        foreach ($result as $storeCode => $item) {
            if ($storeCode === 'x-default') {
                continue;
            }

            if ($storeCode !== $urlRewriteStoreCode) {
                continue;
            }

            $store = $this->storeManager->getStore($storeCode);

            $category = $this->categoryFactory->create()->setStoreId($store->getId())->load($categoryId);

            if (!$category->getData('canonical_url') || $category->getData('canonical_url') === 'default') {
                continue;
            }

            $result = [];
        }

        return $result;
    }
}
